﻿using UnityEngine;
using System.Collections;
using System.Collections.Generic;

public class Recorder : MonoBehaviour {

	public List<Recording> recordings;
	
	public int currentRecording = 0;
	
	public Transform recordTarget;
	public Transform recordTargetGun;
	
	public Transform prefabUnit;
	
	float timeLast = 0f;
	
	int maxSlots = 4;
	
	public bool isRecording = false;

	Runner runner;
	
	void Awake() {
	
		recordings = new List<Recording>();
		
		for (int i = 0; i < maxSlots; i++) {
			recordings.Add( new Recording() );
		}
		
		runner = GameObject.Find("Runner").GetComponent<Runner>();
	}

	// Use this for initialization
	void Start () {		
	}
	
	
	void OnMouseDown() {
		
	}

	public Recording CurrentRecording() {
		return recordings[ currentRecording ];
	}
	
	public Recording RecordingById( int inId) {
		return recordings[ inId ];
	}
	
	
	// Update is called once per frame
	void Update () {
		
		if (Input.GetKeyDown(KeyCode.Alpha1)) {
			Debug.Log ("Playing 1");
			StartRecording( 0 );
		}
		
		if (Input.GetKeyDown(KeyCode.Alpha2)) {
			Debug.Log ("Playing 2");
			StartRecording( 1 );
		}
		
		if (Input.GetKeyDown(KeyCode.Alpha3)) {
			Debug.Log ("Playing 3");
			StartRecording( 2 );
		}
		
		if (Input.GetKeyDown(KeyCode.Alpha4)) {
			Debug.Log ("Playing 4");
			StartRecording( 3 );
		}
		
		if (Input.GetKeyDown(KeyCode.Alpha5)) {
			Debug.Log ("Playing All");
			PlaybackAll();
		}
		
		if ( isRecording && Time.time - timeLast > 0.1f) {
			timeLast = Time.time;
			RecordCurrent();
		}
	}
	
	public void PlaybackAll() {
		isRecording = false;
		StopAllPlayback();
		StartPlaybacks(5);
	}
	
	public void StartRecording(int inPlayback = 0) {
	
		// Reset player position
		recordTarget.position = GameObject.Find("SpawnPoint").transform.position;
	
		isRecording = true;
		
		Screen.lockCursor = true;
		currentRecording = inPlayback;
		recordings[currentRecording].RecordClear();
		
		Debug.Log("* Stopping all playbacks");
		StopAllPlayback();
		
		StartPlaybacks( inPlayback );
	}

	// Plays all except this number
	void StartPlaybacks( int inPlayback ) {
		
		runner.RestoreEnemies();
		
		Debug.Log("* Looping new playbacks");
		for (int i = 0; i < recordings.Count; i++) {
			if (i == inPlayback) { continue; }
			
			StartPlayback( i );
		}
	}	
	
	// Starts playback of everything EXCEPT the one specified
	public void StartPlayback(int inPlayback = 0) {
		StartCoroutine( RecordingPlayback( inPlayback ) );
	}
	
	public void StopAllPlayback() {
	
		StopAllCoroutines();
		for (int i=0; i < recordings.Count; i++) {
			recordings[ i ].isPlaying = false;
		}
	
		// Clear existing playback objs
		int childs = transform.childCount;
		for (int i = childs - 1; i >= 0; i--) {
			GameObject.Destroy(transform.GetChild(i).gameObject);
		}
	}
	
	
	IEnumerator RecordingPlayback( int inPlayback ) {
		
		// Debug.Log ("* Starting playback of " + inPlayback);
		List<RecordingStep> aRecord = recordings[ inPlayback ].RecordGet();
		
		if (aRecord.Count < 3) {
			recordings[ inPlayback ].isPlaying = false;
			Debug.Log("* No recording data for " + inPlayback);
			yield break;
		}
		
		recordings[ inPlayback ].isPlaying = true;
		
		Transform playbackTarget = Instantiate(prefabUnit) as Transform;
		Transform playbackTargetGun = playbackTarget.Find("Gun") as Transform;
		
		playbackTarget.parent = transform;
	}
	
	
}
