﻿using UnityEngine;
using System.Collections;
using System.Collections.Generic;

public class Runner : MonoBehaviour {

	public Transform prefabEnemy;
	
	List<Transform> enemies;
	Transform enemyParent;
	int maxEnemies = 8;
	
	public bool GuiEnabled = true;
	
	Transform cam;
	GameObject camGui;
	
	GameObject player;
	
	public GUIText txtPlan1;
	public GUIText txtPlan2;
	public GUIText txtPlan3;
	public GUIText txtPlan4;
	
	public GUIText statShots;
	public GUIText statKills;
	
	public GUITexture btnPlan1;
	public GUITexture btnPlan2;
	public GUITexture btnPlan3;
	public GUITexture btnPlan4;
	public GUITexture btnReset;
	public GUITexture btnExecute;
	
	Recorder recorder;

	// Use this for initialization
	void Awake () {
		enemies = new List<Transform>();
		enemyParent = GameObject.Find("Enemies").transform;
		
		recorder = GameObject.Find("Recorder").GetComponent<Recorder>();
		
		cam = Camera.main.transform;
		camGui = GameObject.Find("CameraGui");
		player = GameObject.Find("Player");
	}
	
	void Start () {
		CreateEnemies();
		
		ShowGui();
	}
	
	void OnGUI() {
		txtPlan1.text = "Shots: " + recorder.RecordingById(0).statShots + "; Kills: " + recorder.RecordingById(0).statKills;
		txtPlan2.text = "Shots: " + recorder.RecordingById(1).statShots + "; Kills: " + recorder.RecordingById(1).statKills;
		txtPlan3.text = "Shots: " + recorder.RecordingById(2).statShots + "; Kills: " + recorder.RecordingById(2).statKills;
		txtPlan4.text = "Shots: " + recorder.RecordingById(3).statShots + "; Kills: " + recorder.RecordingById(3).statKills;
		
		statShots.text = "Shots Fired: " + recorder.CurrentRecording().statShots;
		statKills.text = "Kills: " + recorder.CurrentRecording().statKills;
	}
	
	
	void Update() {
		if (GuiEnabled) {
			
			if (Input.GetMouseButtonDown(0)) {
				if (btnPlan1.HitTest(Input.mousePosition)) {
					HideGui();
					recorder.StartRecording( 0 );
					Invoke("ShowGui", 10f);
				}
				if (btnPlan2.HitTest(Input.mousePosition)) {
					HideGui();
					recorder.StartRecording( 1 );
					Invoke("ShowGui", 10f);
				}
				if (btnPlan3.HitTest(Input.mousePosition)) {
					HideGui();
					recorder.StartRecording( 2 );
					Invoke("ShowGui", 10f);
				}
				if (btnPlan4.HitTest(Input.mousePosition)) {
					HideGui();
					recorder.StartRecording( 3 );
					Invoke("ShowGui", 10f);
				}
				if (btnExecute.HitTest(Input.mousePosition)) {
					HideGui();
					cam.FindChild("Gun").renderer.enabled = false;
					recorder.PlaybackAll();
					Invoke("ShowGui", 12f);
				}
				if (btnReset.HitTest(Input.mousePosition)) {
					Application.LoadLevel("Main");
				}
			}		
		}
	}
	

	public void ShowGui ()	{
		GuiEnabled = true;
		recorder.isRecording = false;
		camGui.SetActive(true);
		player.SetActive(false);
		Screen.lockCursor = false;
		recorder.PlaybackAll();
	}
	
	public void HideGui ()	{
		GuiEnabled = false;
		camGui.SetActive(false);
		player.SetActive(true);
		cam.FindChild("Gun").renderer.enabled = true;
		Screen.lockCursor = true;
	}
	
	
	public void CreateEnemies() {
		while (enemies.Count < maxEnemies) {
			
			Vector3 spawnPos = RandomSpawnLoc();
			
			CreateEnemy( spawnPos, Quaternion.Euler(0, Random.Range(-180, 180), 0) );
		}
	}
	
	// Move back to original pos
	public void RestoreEnemies() {
		foreach (Transform anEnemy in enemies) {
			anEnemy.GetComponent<Unit>().RestorePosition();
		}
	}
	
	
	public Transform CreateEnemy( Vector3 inPos, Quaternion inRot ) {
		
		Transform anEnemy = Instantiate( prefabEnemy, inPos, inRot ) as Transform;
		anEnemy.parent = enemyParent;
		anEnemy.GetComponent<Unit>().originalPos = inPos;
		anEnemy.GetComponent<Unit>().originalRot = inRot;
		enemies.Add(anEnemy);
		
		return anEnemy;
	
	}
	
	
	Vector3 RandomSpawnLoc() {
		
		bool found = false;
		Vector3 res = Vector3.zero;
		RaycastHit hit;
		
		while (!found) {
			
			res = new Vector3( Random.Range(-6f, -55f), 16f, Random.Range( -5f, 42f ) );
			if (Physics.Raycast( res, Vector3.down, out hit )) {
				// Debug.Log ("* Hit " + hit.collider.tag );
				if (hit.collider.CompareTag("Ground")) {
					found = true;
					res = hit.point;
					res.y = 1.2f;
				}
			}
			
		}
		
		return res;
	}
	
	public void PlayBlip() {
		audio.Play();
	}
}
